SUBROUTINE TF_MISFITS (S, SREF, DT, MT, FMIN, FMAX, NF_TF,             &
                                TFEM,  TFPM,                           &
                                 TEM,   TPM,                           &
                                 FEM,   FPM,                           &
                                  EM,    PM,  CWT_REF  )

  USE GLOBAL, ONLY: WP, PI, W0, KN, NF

  IMPLICIT NONE

!----------------------------------------------------------------------

  INTEGER , INTENT(IN) :: MT, NF_TF
  REAL(WP), INTENT(IN) :: DT, FMIN, FMAX
  
  REAL(WP), DIMENSION (1:MT)   , INTENT(IN)  :: S, SREF
  
  REAL(WP),                      INTENT(OUT) ::  EM, PM
  REAL(WP), DIMENSION (1:MT   ), INTENT(OUT) :: TEM, TPM
  REAL(WP), DIMENSION (1:NF_TF), INTENT(OUT) :: FEM, FPM

  REAL(WP), DIMENSION (1:MT,1:NF_TF), INTENT(OUT) ::  TFEM, TFPM,      &
                                                      CWT_REF

  INTEGER            :: I, L

  REAL(WP)           :: NOMSE, NOMSP, DENOMS, DF, FF, MAXDENOM, MAXTF
  
  REAL(WP), DIMENSION(1:MT,1:NF_TF)    :: DE , DP

  REAL(WP), DIMENSION(1:MAX(MT,NF_TF)) :: DET, DPT, DEF, DPF
                                          
  COMPLEX , DIMENSION(NF)           :: SS, SS_REF
  COMPLEX , DIMENSION(1:MT,1:NF_TF) :: WV, WV_REF

!----------------------------------------------------------------------

  INTERFACE 

    FUNCTION CWT(F_V, MT, NF_TF, DFA, FF, FMIN)
      USE GLOBAL, ONLY: WP
      IMPLICIT NONE
      INTEGER , INTENT(IN)    :: MT, NF_TF
      REAL(WP), INTENT(IN)    :: DFA, FF, FMIN
      COMPLEX , INTENT(INOUT), DIMENSION(:) :: F_V
      COMPLEX, DIMENSION (1:MT, 1:NF_TF) :: CWT
    END FUNCTION CWT
    
  END INTERFACE

!----------------------------------------------------------------------

! COMPUTATION OF CWT

  DF  = 1./(DT*REAL(NF))

  FF = EXP( LOG(FMAX/FMIN) / REAL(NF_TF - 1) )
   
  SS     = CMPLX(0.,0.)
  SS_REF = CMPLX(0.,0.)
  DO I = 1, MT
    SS     (I) = CMPLX( S   (I), 0. )
    SS_REF (I) = CMPLX( SREF(I), 0. )
  END DO
  CALL FCOOLR (KN, SS    , -1.)
  CALL FCOOLR (KN, SS_REF, -1.)

  WV     (1:MT,1:NF_TF) = CWT(SS    , MT ,NF_TF, DF, FF, FMIN)
  WV_REF (1:MT,1:NF_TF) = CWT(SS_REF, MT, NF_TF, DF, FF, FMIN)

  CWT_REF = ABS(WV_REF)
  
! COMPUTATION OF TFEM AND TFPM

  MAXTF = MAXVAL(ABS(WV_REF(1:MT,1:NF_TF)))
        
  DE =               ( ABS(WV) - ABS(WV_REF) )
  DP = ABS(WV_REF) * (ATAN2( IMAG(WV/WV_REF) , REAL(WV/WV_REF) )) / PI
   
  TFEM = DE / MAXTF
  TFPM = DP / MAXTF
        
! COMPUTATION OF AUXILIARY VARIABLES

  DO I = 1, MT
    DET  (I) = 0.
    DPT  (I) = 0.
    DO L = 1, NF_TF
      DET (I) = DET (I) +     DE(I,L)
      DPT (I) = DPT (I) +     DP(I,L)
    END DO
    DET (I) = DET (I)/REAL(NF_TF)
    DPT (I) = DPT (I)/REAL(NF_TF)
  END DO

  DO L = 1, NF_TF
    DEF   (L) = 0.
    DPF   (L) = 0.
    DO I = 1, MT
      DEF (L) = DEF (L) +     DE(I,L)
      DPF (L) = DPF (L) +     DP(I,L)
    END DO
    DEF (L) = DEF (L)/REAL(MT)
    DPF (L) = DPF (L)/REAL(MT)
  END DO  
  
! COMPUTATION OF TEM

  MAXDENOM = 0.
  DO I = 1, MT
    DENOMS = 0.
    DO L = 1, NF_TF
      DENOMS = DENOMS + ABS(WV_REF(I,L))
    END DO
    DENOMS = DENOMS / REAL(NF_TF)
    IF ( DENOMS > MAXDENOM ) MAXDENOM = DENOMS
  END DO

  DO I = 1, MT
    TEM(I) = DET (I) / MAXDENOM
  END DO

! COMPUTATION OF TPM

  DO I = 1, MT
    TPM(I) = DPT (I) / MAXDENOM
  END DO

! COMPUTATION OF FEM

  MAXDENOM = 0.
  DO L = 1, NF_TF
    DENOMS = 0.
    DO I = 1, MT
      DENOMS = DENOMS + ABS(WV_REF(I,L))
    END DO
    DENOMS = DENOMS / REAL(MT)
    IF ( DENOMS > MAXDENOM  ) MAXDENOM  = DENOMS
  END DO

  DO L = 1, NF_TF
    FEM(L) = DEF (L) / MAXDENOM
  END DO

! COMPUTATION OF FPM

  DO L = 1, NF_TF
    FPM(L) = DPF (L) / MAXDENOM
  END DO

! COMPUTATION OF EM AND PM
 
  NOMSE  = 0.
  NOMSP  = 0.
  DENOMS = 0.

  DO I = 1, MT
    DO L = 1, NF_TF

      NOMSE  = NOMSE  + ABS(DE    (I,L))*ABS(DE    (I,L))
      NOMSP  = NOMSP  + ABS(DP    (I,L))*ABS(DP    (I,L))
      DENOMS = DENOMS + ABS(WV_REF(I,L))*ABS(WV_REF(I,L))

    END DO
  END DO
  
  EM = SQRT ( NOMSE / DENOMS )
  PM = SQRT ( NOMSP / DENOMS )


END SUBROUTINE  TF_MISFITS